import java.io.*;
import java.nio.file.*;
import java.nio.file.attribute.*;
import java.util.*;

class FilesAndFolders {
   
	static void getSpaceOfStore(FileStore store) throws IOException {
		final long MEGA = 1024*1024;
		long gesamt = store.getTotalSpace()/MEGA;
		long belegt = (store.getTotalSpace()-store.getUsableSpace())/MEGA;
		long frei = store.getUsableSpace()/MEGA;
		System.out.format("%-20s %12d %12d %12d\n", store, gesamt, belegt, frei);
	}
	
	public static void main(String[] args) {
		byte[] arr = {1, 2, 3};

		Path dir = Paths.get("U:", "Eigene Dateien", "Java", "io");
		
		// Sich später auf ein Testergebnis zu verlassen, ist unsicher (TOCTTOU-Falle).
		if (Files.exists(dir))
			System.out.println(dir+" existiert bereits.");
		else
			if (Files.notExists(dir))
				System.out.println(dir+" existiert noch nicht");
			else
				System.out.println(dir+" hat einen unbekannten Status.");
		
		// Verzeichnis ggf. erstellen
		try {
			Files.createDirectories(dir);
		} catch (FileAlreadyExistsException ae) {
			System.err.println(dir + " existiert, ist aber kein Verzeichnis.");
			System.exit(1);
		} catch (IOException ioe) {
			System.err.println(ioe);
			System.exit(1);
		}

		// Datei explizit erstellen
		Path file = dir.resolve("Ausgabe.txt");
		try {
			Files.createFile(file);
		} catch (FileAlreadyExistsException ae) {
			System.err.println(file + " existiert bereits.");
		} catch (IOException ioe) {
			System.err.println(ioe);
		}

		try {
			Files.createTempFile(null, null);
		} catch (IOException ioe) {
			System.err.println(ioe);
		}		
		
		// Daten in die Ausgabedatei schreiben
		try {
			Files.write(file,  arr);
			System.out.println("Daten geschrieben in Datei "+file);
		} catch (IOException e) {
			System.out.println("Fehler beim Schreiben in Datei "+file);
		}

		// Über ein Dateisystemobjekt informieren
		System.out.println("\nEigenschaften von    " + file);
//		file = Paths.get("E:", "Daten", "LWU", "Eigene Dateien", "Java", "NioFile", "Ort", "Ausgabe.txt");
		try {
//			BasicFileAttributes attr = Files.readAttributes(file, BasicFileAttributes.class);
//			BasicFileAttributes attr = Files.readAttributes(name, BasicFileAttributes.class, LinkOption.NOFOLLOW_LINKS);
//			System.out.println(" Größe in Bytes:   " + attr.size());
			System.out.println(" Größe in Bytes:     " + Files.size(file));
			System.out.println(" Letzte Änderung:    " + Files.getLastModifiedTime(file));
			System.out.println(" Datei:              " + Files.isRegularFile(file));
			System.out.println(" Verzeichnis:        " + Files.isDirectory(file));
			System.out.println(" Schreiben möglich:  " + Files.isWritable(file));
//			System.out.println(" Symb. Link:        " + attr.isSymbolicLink(name));
		} catch (Exception e) {
			System.err.println(e);
		}
		
		// Datum der letzten Modifikation ändern
		try {
			System.out.println("\nDatum der letzten Modifikation setzen\n vorher:    " + Files.getLastModifiedTime(file));
			Calendar cal = Calendar.getInstance();
			cal.set(1980, Calendar.APRIL, 1, 0, 0, 0);
			Files.setLastModifiedTime(file, FileTime.fromMillis(cal.getTimeInMillis()));
			System.out.println(" nachher:   " + Files.getLastModifiedTime(file));
		} catch (Exception e) {
			System.err.println(e);
		}

		// Datei kopieren
		try {
			Files.copy(file, dir.resolve("Kopie.txt"));
		} catch (Exception e) {
			System.err.println(e);
		}
		
		// Ordner kopieren
		try {
			Files.copy(dir, dir.resolveSibling("io2"));
		} catch (Exception e) {
			System.err.println(e);
		}
		
		// Datei umbenennen
		try {
			Files.move(file, file.resolveSibling("Umbenannt.txt"));
			System.out.println("\nDatei " + file + " umbenannt");
		} catch (Exception e) {
			System.err.println(e);
		}

		// Datei verschieben
		try {
			Files.move(dir.resolve("Umbenannt.txt"), dir.getParent().resolve("Verschoben.txt"), StandardCopyOption.REPLACE_EXISTING);
			System.out.println("\nDatei " + dir.resolve("Umbenannt.txt") + " verschoben");
		} catch (Exception e) {
			System.err.println(e);
		}

		// Verzeichnisinhalte auflisten
		System.out.println("\nDateien im Verzeichnis " + dir);
		try (DirectoryStream<Path> ds = Files.newDirectoryStream(dir)) {
			for (Path path: ds)
				System.out.println(" " + path.getFileName());
		} catch (Exception e) {
			System.err.println(e);
		}
		
		// Dateien löschen
		try (DirectoryStream<Path> ds = Files.newDirectoryStream(dir)) {
		//			Files.delete(Paths.get("U:", "Eigene Dateien", "Java", "io", "Kopie.txt"));
			System.out.println("\nDateien löschen:");
			for (Path path: ds) {
				Files.delete(path);
				System.out.println(" " + path.getFileName());
			}
		} catch (Exception e) {
			System.err.println(e);
		}

		// Verzeichnisse löschen
		try {
			Files.delete(dir);
			Files.delete(dir.resolveSibling("io2"));
			System.out.println("\nVerzeichnis gelöscht");			
		} catch (Exception e) {
			System.err.println(e);
		}
		
		// Nutzbares Speichervolumen ermitteln
        System.out.format("\n%-20s %12s %12s %12s\n", "Laufwerk", "Gesamt (MB)", "Belegt (MB)", "Frei (MB)");
        FileSystem fs = FileSystems.getDefault();
		try {
			for (FileStore store: fs.getFileStores())
				getSpaceOfStore(store);
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
	}
}