// Bruch implements Comparable<Bruch>

package de.zimk.matrain.br;
import de.zimk.util.conio.Simput;
import static java.lang.Math.abs;

public class Bruch implements Comparable<Bruch> {
	private int zaehler;          // wird automatisch mit 0 initialisiert
	private int nenner = 1;       // wird manuell mit 1 initialisiert
	private String etikett = "";  // die Ref.typ-Init. auf null wird ersetzt, siehe Text

	static private int anzahl;    // wird automatisch mit 0 initialisiert

	public Bruch(int zpar, int npar, String epar) {
		setzeZaehler(zpar);
		setzeNenner(npar);
		setzeEtikett(epar);
		anzahl++;
	}
	
	public Bruch() {anzahl++;}

	public void setzeZaehler(int zpar) {zaehler = zpar;}

	public boolean setzeNenner(int n) {
		if (n != 0)	{
			nenner = n;
			return true;
		} else
			return false;
	}

	public void setzeEtikett(String epar) {
		if (epar.length() <= 40)
			etikett = epar;
		else
			etikett = epar.substring(0, 40);
	}
	
	public int gibZaehler() {return zaehler;}

	public int gibNenner()	{return nenner;}

	public String gibEtikett()	{return etikett;}

	public void kuerze() {
		if (zaehler != 0) {
			int rest;
			int ggt = abs(zaehler);
			int divisor = abs(nenner);
			do {
				rest = ggt % divisor;
				ggt = divisor;
				divisor = rest;
			} while (rest > 0);			
			zaehler /= ggt;
			nenner  /= ggt;
		} else
			nenner = 1;
	}	

	public void addiere(Bruch b) {
		zaehler = zaehler*b.nenner + b.zaehler*nenner;
		nenner = nenner*b.nenner;
		kuerze();
	}

	public boolean addiere(int zpar, int npar, boolean autokurz) {
		if (npar != 0) {
			zaehler = zaehler*npar + zpar*nenner;
			nenner = nenner*npar;
			if (autokurz)
				kuerze();
			return true;
		} else
			return false;
	}

	public void addiere(Bruch ... bar) {
		for (Bruch b : bar)
			addiere(b);
	}
	
	public void frage() {
		int n;
		do {
			System.out.print("Zaehler: ");
			setzeZaehler(Simput.gint());
		} while (Simput.checkError());
		do {
			System.out.print("Nenner : ");
			n = Simput.gint();
			if (n == 0 && !Simput.checkError())
				System.out.println("Der Nenner darf nicht Null werden!\n");
		} while (n == 0);
		setzeNenner(n);
	}

	public void zeige() {
		String luecke = "";
		int el = etikett.length();
		for (int i=1; i<=el; i++)
			luecke = luecke + " ";
		System.out.println(" " + luecke  + "   " + zaehler + "\n" +
		                   " " + etikett + " -----\n" +
		                   " " + luecke  + "   " + nenner + "\n");
	}

	public void dupliziere(Bruch bc) {
		bc.zaehler = zaehler;
		bc.nenner = nenner;
		bc.etikett = etikett;
	}
	public Bruch klone() {
		anzahl++;
		return new Bruch(zaehler, nenner, etikett);
	}

	public static int hanz() {return anzahl;}

	public int compareTo(Bruch b) {
		int sign = Integer.signum(zaehler)*Integer.signum(nenner);
		int signb = Integer.signum(b.zaehler)*Integer.signum(b.nenner);
		int temp = abs(zaehler*b.nenner)*sign - abs(b.zaehler*nenner)*signb;
		if (temp < 0)
			return -1;
		else
			if (temp == 0)
				return 0;
			else
				return 1;
	}
}